@@ -11,24 +11,32 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import sysconfig
-from .. import mesonlib, dependencies
+from .. import mesonlib
 
-from . import ExtensionModule
-from mesonbuild.modules import ModuleReturnValue
-from . import permittedSnippetKwargs
-from ..interpreterbase import noKwargs
+from . import ExtensionModule, ModuleInfo
+from ..interpreterbase import typed_pos_args, noPosargs, noKwargs, permittedKwargs
 from ..build import known_shmod_kwargs
+from ..programs import ExternalProgram
 
 
 class Python3Module(ExtensionModule):
+
+    INFO = ModuleInfo('python3', '0.38.0', deprecated='0.48.0')
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.snippets.add('extension_module')
+        self.methods.update({
+            'extension_module': self.extension_module,
+            'find_python': self.find_python,
+            'language_version': self.language_version,
+            'sysconfig_path': self.sysconfig_path,
+        })
 
-    @permittedSnippetKwargs(known_shmod_kwargs)
-    def extension_module(self, interpreter, state, args, kwargs):
+    @permittedKwargs(known_shmod_kwargs)
+    def extension_module(self, state, args, kwargs):
         if 'name_prefix' in kwargs:
             raise mesonlib.MesonException('Name_prefix is set automatically, specifying it is forbidden.')
         if 'name_suffix' in kwargs:
@@ -44,29 +52,33 @@ class Python3Module(ExtensionModule):
             suffix = []
         kwargs['name_prefix'] = ''
         kwargs['name_suffix'] = suffix
-        return interpreter.func_shared_module(None, args, kwargs)
+        return self.interpreter.func_shared_module(None, args, kwargs)
 
+    @noPosargs
     @noKwargs
     def find_python(self, state, args, kwargs):
-        py3 = dependencies.ExternalProgram('python3', mesonlib.python_command, silent=True)
-        return ModuleReturnValue(py3, [py3])
+        command = state.environment.lookup_binary_entry(mesonlib.MachineChoice.HOST, 'python3')
+        if command is not None:
+            py3 = ExternalProgram.from_entry('python3', command)
+        else:
+            py3 = ExternalProgram('python3', mesonlib.python_command, silent=True)
+        return py3
 
+    @noPosargs
     @noKwargs
     def language_version(self, state, args, kwargs):
-        return ModuleReturnValue(sysconfig.get_python_version(), [])
+        return sysconfig.get_python_version()
 
     @noKwargs
+    @typed_pos_args('python3.sysconfig_path', str)
     def sysconfig_path(self, state, args, kwargs):
-        if len(args) != 1:
-            raise mesonlib.MesonException('sysconfig_path() requires passing the name of path to get.')
         path_name = args[0]
         valid_names = sysconfig.get_path_names()
         if path_name not in valid_names:
-            raise mesonlib.MesonException('{} is not a valid path name {}.'.format(path_name, valid_names))
+            raise mesonlib.MesonException(f'{path_name} is not a valid path name {valid_names}.')
 
         # Get a relative path without a prefix, e.g. lib/python3.6/site-packages
-        path = sysconfig.get_path(path_name, vars={'base': '', 'platbase': '', 'installed_base': ''})[1:]
-        return ModuleReturnValue(path, [])
+        return sysconfig.get_path(path_name, vars={'base': '', 'platbase': '', 'installed_base': ''})[1:]
 
 
 def initialize(*args, **kwargs):
